<?php

namespace Drupal\danse_content\Plugin\Danse;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\danse_content\Topic\TopicInterface;

/**
 * Trait ContentSettingsTrait
 *
 * @package Drupal\danse_content\Plugin\Danse
 */
trait ContentSettingsTrait {

  use StringTranslationTrait;

  /**
   * @param string $entity_type_id
   * @param string $bundle
   *
   * @return string
   */
  protected function configId(string $entity_type_id, $bundle = NULL): string {
    return 'danse.settings.content.' . $entity_type_id . '.' . ($bundle ?? $entity_type_id);
  }


  /**
   * @return array
   */
  protected function settings(): array {
    return [
      self::SETTING_CREATE_EVENT => [
        'label' => $this->t('Create event'),
      ],
    ];
  }

  /**
   * @return array
   */
  protected function defaultSettingLabels(): array {
    return [
      'access settings' => $this->t('Access to settings'),
      'push' => $this->t('Default value for push'),
      'force' => $this->t('Default value for force'),
      'silent' => $this->t('Default value for silent'),
    ];
  }

  /**
   * @param array $roles
   * @param \Drupal\Core\Config\ImmutableConfig $config
   * @param TopicInterface $topic
   * @param bool $forEdit
   *
   * @return array
   */
  protected function defaultSettings(array $roles, ImmutableConfig $config, TopicInterface $topic, bool $forEdit): array {
    return [
      'access settings' => [
        '#type' => 'select',
        '#options' => $roles,
        '#multiple' => TRUE,
        '#default_value' => $config->get($topic->getSettingKey('access settings')) ?? [],
        '#title' => $this->t('Access to settings'),
        '#title_display' => 'hidden',
        '#access' => !$forEdit,
      ],
      'push' => [
        '#type' => 'checkbox',
        '#default_value' => $config->get($topic->getSettingKey('push')) ?? TRUE,
        '#title' => $this->t('Create push notification'),
        '#title_display' => $forEdit ? 'after' : 'hidden',
      ],
      'force' => [
        '#type' => 'checkbox',
        '#default_value' => $config->get($topic->getSettingKey('force')) ?? FALSE,
        '#title' => $this->t('Force push even if this creates duplicate notification'),
        '#title_display' => $forEdit ? 'after' : 'hidden',
      ],
      'silent' => [
        '#type' => 'checkbox',
        '#default_value' => $config->get($topic->getSettingKey('silent')) ?? FALSE,
        '#title' => $this->t('Make this silent and do not create any notifications'),
        '#title_display' => $forEdit ? 'after' : 'hidden',
        '#description' => $this->t('If this is checked, the other options have no effect anymore.'),
      ],
    ];
  }

}
