<?php

namespace Drupal\danse_content\Plugin\views\field;

use Drupal\danse_content\Service;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a field for the entity subscription widget.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("danse_subscription")
 */
class Subscription extends FieldPluginBase {

  /**
   * @var \Drupal\danse_content\Service
   */
  protected Service $service;

  /**
   * Subscription constructor.
   *
   * @param array $configuration
   * @param $plugin_id
   * @param $plugin_definition
   * @param \Drupal\danse_content\Service $service
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Service $service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->service = $service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('danse_content.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function usesGroupBy(): bool {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function query(): void {
    // Nothing to do.
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    return _danse_content_service()->widget($values->_entity);
  }

}
