<?php

namespace Drupal\danse_content;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\danse\Service as DanseService;
use Drupal\danse_content\Plugin\Danse\Content;
use Drupal\user\UserDataInterface;

/**
 * Logger service to write messages and exceptions to an external service.
 */
class Service {

  use StringTranslationTrait;

  /**
   * @var \Drupal\danse\Service
   */
  protected $service;

  /**
   * @var \Drupal\user\UserDataInterface
   */
  protected $userData;

  /**
   * @var \Drupal\danse_content\Plugin\Danse\Content
   */
  protected $plugin;

  /**
   * @param \Drupal\danse\Service $service
   * @param \Drupal\user\UserDataInterface $user_data
   */
  public function __construct(DanseService $service, UserDataInterface $user_data) {
    $this->service = $service;
    $this->userData = $user_data;
  }

  /**
   * @return \Drupal\danse_content\Plugin\Danse\Content
   */
  public function getPlugin(): Content {
    if ($this->plugin === NULL) {
      $this->plugin = $this->service->getPluginInstance('content');
    }
    return $this->plugin;
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *
   * @return string
   */
  public function widgetId(ContentEntityInterface $entity): string {
    return 'danse-widget-' . $entity->getEntityTypeId() . '-' . $entity->id();
  }

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *
   * @return array
   */
  public function widget(ContentEntityInterface $entity): array {
    $operations = [];

    $subscribeItems = [];
    $unsubscribeItems = [];
    foreach ($this->getPlugin()->subscriptionOperations($entity) as $subscriptionOperation) {
      if ($subscriptionOperation->isSubscribeMode()) {
        $subscribeItems[] = $subscriptionOperation;
      }
      else {
        $unsubscribeItems[] = $subscriptionOperation;
      }
    }
    if (!empty($subscribeItems) || !empty($unsubscribeItems)) {
      foreach ([$subscribeItems, $unsubscribeItems] as $items) {
        /** @var \Drupal\danse_content\SubscriptionOperation $item */
        foreach ($items as $item) {
          $operations[] = $item->operationItem();
        }
      }
    }

    if (!empty($operations)) {
      return [
        '#theme_wrappers' => ['danse_content_subscription_wrapper'],
        '#id' => $this->widgetId($entity),
        '#dropbutton_type' => 'danse-widget-' . $entity->getEntityTypeId(),
        '#type' => 'operations',
        '#subtype' => 'danse__' . $entity->getEntityTypeId(),
        '#links' => $operations,
        '#cache' => [
          'max-age' => 0,
        ],
      ];
    }
    return [];
  }

}
