<?php

namespace Drupal\danse_content;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\danse_content\Topic\TopicInterface;

/**
 * Class SubscriptionOption
 *
 * @package Drupal\danse_content
 */
class SubscriptionOperation {

  use StringTranslationTrait;

  public const MODE_SUBSCRIBE = 1;
  public const MODE_UNSUBSCRIBE = 0;

  public const SUBSCRIPTION_MODE_ENTITY_TYPE = '0';
  public const SUBSCRIPTION_MODE_ENTITY = '1';
  public const SUBSCRIPTION_MODE_RELATED_ENTITY = '2';

  /**
   * @var int
   */
  protected int $mode;


  /**
   * @var \Drupal\Core\Entity\ContentEntityInterface
   */
  protected ContentEntityInterface $entity;

  /**
   * @var string
   */
  protected string $key;

  /**
   * @var \Drupal\danse_content\Topic\TopicInterface
   */
  protected TopicInterface $topic;

  /**
   * @var string
   */
  protected string $subscriptionMode;

  /**
   * SubscriptionOption constructor.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param string $key
   * @param \Drupal\danse_content\Topic\TopicInterface $topic
   * @param string $subscription_mode
   */
  public function __construct(ContentEntityInterface $entity, string $key, TopicInterface $topic, string $subscription_mode) {
    $this->mode = self::MODE_SUBSCRIBE;
    $this->entity = $entity;
    $this->key = $key;
    $this->topic = $topic;
    $this->subscriptionMode = $subscription_mode;
  }

  /**
   * @return bool
   */
  public function isSubscribeMode(): bool {
    return $this->mode === self::MODE_SUBSCRIBE;
  }

  /**
   * @return \Drupal\danse_content\SubscriptionOperation
   */
  public function setSubscribeMode(): SubscriptionOperation {
    $this->mode = self::MODE_SUBSCRIBE;
    return $this;
  }

  /**
   * @return \Drupal\danse_content\SubscriptionOperation
   */
  public function setUnsubscribeMode(): SubscriptionOperation {
    $this->mode = self::MODE_UNSUBSCRIBE;
    return $this;
  }

  /**
   * @return array
   */
  public function operationItem(): array {
    if ($this->isSubscribeMode()) {
      $routeName = 'danse_content.api.subscribe';
      $class = 'subscribe';
    }
    else {
      $routeName = 'danse_content.api.unsubscribe';
      $class = 'unsubscribe';
    }
    return [
      'title' => $this->topic->operationLabel($this->entity, $this->isSubscribeMode(), $this->subscriptionMode),
      'weight' => 0,
      'url' => Url::fromRoute($routeName, [
        'entity_type' => $this->entity->getEntityTypeId(),
        'entity_id' => $this->entity->id(),
        'key' => $this->key,
      ], [
        'attributes' => [
          'class' => ['danse-subscription-operation', $class, $this->key, 'use-ajax'],
        ],
      ]),
    ];
  }

}
