<?php

namespace Drupal\danse_content\Topic;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Interface TopicInterface
 *
 * @package Drupal\danse_content\Topic
 */
interface TopicInterface {

  public const CREATE = 'create';

  public const UPDATE = 'update';

  public const DELETE = 'delete';

  public const PUBLISH = 'publish';

  public const UNPUBLISH = 'unpublish';

  public const COMMENT = 'comment';

  public function id(): string;

  /**
   * @param string $key
   *
   * @return string
   */
  public function getSettingKey(string $key): string;

  /**
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   */
  public function settingsFormLabel(): TranslatableMarkup;

  /**
   * @return bool
   */
  public function accessToDefaultSettings(): bool;

  /**
   * @return array
   */
  public function excludedOnEntityTypes(): array;

  /**
   * @return array
   */
  public function dependencies(): array;

  /**
   * @param string $entity_type_id
   *
   * @return bool
   */
  public function assert(string $entity_type_id): bool;

  /**
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   * @param bool $subscribe
   * @param int $subscription_mode
   *
   * @return string
   */
  public function operationLabel(ContentEntityInterface $entity, bool $subscribe, int $subscription_mode): string;

  public function actionForLabel(): string;

}
