<?php

namespace Drupal\danse_form\Plugin\Danse;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\danse\Entity\EventInterface;
use Drupal\danse\PayloadInterface;
use Drupal\danse\PluginBase;
use Drupal\danse_form\Payload;

/**
 * Plugin implementation of DANSE.
 *
 * @Danse(
 *   id = "form",
 *   label = @Translation("Form"),
 *   description = @Translation("Manages all form events.")
 * )
 */
class Form extends PluginBase {

  public const TOPIC_SUBMIT = 'submit';

  /**
   * {@inheritdoc}
   */
  public function assertPayload(PayloadInterface $payload): bool {
    return $payload instanceof Payload;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedSubscriptions($roles): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(&$form, FormStateInterface $form_state) {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(EventInterface $event): Url {
    return Url::fromRoute('<front>');
  }

  /**
   * @param string $topic
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return \Drupal\danse\Entity\EventInterface|null
   */
  public function createFormEvent(string $topic, array $form, FormStateInterface $form_state): ?EventInterface {
    $message = $form['#form_id'];
    $payload = new Payload($form, $form_state);
    return $this->createEvent($topic, $message, $payload);
  }

}
