<?php

namespace Drupal\danse_user\Plugin\Danse;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\danse\Entity\EventInterface;
use Drupal\danse\PayloadInterface;
use Drupal\danse\PluginBase;
use Drupal\danse_user\RolePayload;
use Drupal\user\RoleInterface;

/**
 * Plugin implementation of DANSE.
 *
 * @Danse(
 *   id = "role",
 *   label = @Translation("Role"),
 *   description = @Translation("Manages all role events.")
 * )
 */
class Role extends PluginBase {

  /**
   * {@inheritdoc}
   */
  public function assertPayload(PayloadInterface $payload): bool {
    return $payload instanceof RolePayload;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedSubscriptions($roles): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(&$form, FormStateInterface $form_state) {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(EventInterface $event): Url {
    return Url::fromRoute('<front>');
  }

  /**
   * @param array $oldPermissions
   * @param \Drupal\user\RoleInterface $role
   */
  public function createChangedPermissionsEvents(array $oldPermissions, RoleInterface $role): void {
    $payload = new RolePayload($role);
    $newPermissions = $role->getPermissions();
    foreach (array_diff($oldPermissions, $newPermissions) as $permission) {
      $this->createEvent(RolePayload::TOPIC_PERMISSION_REMOVED, $permission, $payload);
    }
    foreach (array_diff($newPermissions, $oldPermissions) as $permission) {
      $this->createEvent(RolePayload::TOPIC_PERMISSION_ADDED, $permission, $payload);
    }
  }

}
