<?php

namespace Drupal\danse_user\Plugin\Danse;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\danse\Entity\EventInterface;
use Drupal\danse\PayloadInterface;
use Drupal\danse\PluginBase;
use Drupal\danse_user\UserPayload;

/**
 * Plugin implementation of DANSE.
 *
 * @Danse(
 *   id = "user",
 *   label = @Translation("User"),
 *   description = @Translation("Manages all user events.")
 * )
 */
class User extends PluginBase {

  /**
   * {@inheritdoc}
   */
  public function assertPayload(PayloadInterface $payload): bool {
    return $payload instanceof UserPayload;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedSubscriptions($roles): array {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(&$form, FormStateInterface $form_state) {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(EventInterface $event): Url {
    return Url::fromRoute('<front>');
  }

  /**
   * @param string $topic
   * @param \Drupal\Core\Session\AccountInterface $account
   *
   * @return \Drupal\danse\Entity\EventInterface|null
   */
  public function createUserEvent(string $topic, AccountInterface $account): ?EventInterface {
    $message = $account->getDisplayName();
    $payload = new UserPayload($account);
    return $this->createEvent($topic, $message, $payload);
  }

  /**
   * @param array $oldRoles
   * @param \Drupal\Core\Session\AccountInterface $account
   */
  public function createChangedRolesEvents(array $oldRoles, AccountInterface $account): void {
    $payload = new UserPayload($account);
    $newRoles = $account->getRoles(TRUE);
    foreach (array_diff($oldRoles, $newRoles) as $role) {
      $this->createEvent(UserPayload::TOPIC_ROLE_REMOVED, $role, $payload);
    }
    foreach (array_diff($newRoles, $oldRoles) as $role) {
      $this->createEvent(UserPayload::TOPIC_ROLE_ADDED, $role, $payload);
    }
  }

}
